module One
  module Backup
    class HapyVM
        attr_reader :id, :info, :name, :disks, :images, :labels, :vm_path
        def initialize(vm, images)
            @images = self._update_image_pool(images)
            @info = vm.to_hash
            @id = @info["VM"]["ID"]
            @name = @info["VM"]["NAME"]
            @labels = self._get_labels(@info["VM"]["USER_TEMPLATE"]["LABELS"])
            @disks = self._get_disks(@info["VM"]["TEMPLATE"]["DISK"])
            @deployment_files = self._locate_deployement()
            @vm_path = self._get_vm_files_path()
            if @info["VM"]["TEMPLATE"]["CONTEXT"]
                @context_disk_id = @info["VM"]["TEMPLATE"]["CONTEXT"]["DISK_ID"]
            end
            @vm = vm
        end
    
        def _get_labels(lbs)
            if lbs
                return lbs.split(",")
            end
            return []
        end
    
        def _get_vm_files_path()
            ds_id = 0
            history = @info["VM"]["HISTORY_RECORDS"]["HISTORY"]
            if history.is_a?(Hash)
                ds_id = @info["VM"]["HISTORY_RECORDS"]["HISTORY"]["DS_ID"]
            elsif history.is_a?(Array)
                ds_id = @info["VM"]["HISTORY_RECORDS"]["HISTORY"][-1]["DS_ID"]
            else
                return nil
            end
            return "#{ONEROOT}/datastores/#{ds_id}/#{@id}"
        end
    
        def _update_image_pool(pool)
            newpool = {}
            newpool.store("pool",pool["pool"])
            pool["pool"].info
            pool["pool"].each do |img|
                infoid = img.to_hash['IMAGE']['ID']
                newpool.store(infoid, HapyImage.new(img))
            end
            return newpool
        end
    
        # Aim: Return a list of disks (HapyDisk)
        # Param: Virtual machine template disk information from VM/TEMPLATE/DISK
        # Note: This param ca be a hash or a list of hash so we need check the information type
        #       in order to provide the correct disk information.
        # Return : Allways a list of HapyDisk objects (even for on disk)
        def _get_disks(dsinfo)
            disks = []
            if dsinfo.is_a?(Array)
                dsinfo.each do |ds|
                    disks.append(HapyDisk.new(ds))
                end
            else
                disks.append(HapyDisk.new(dsinfo))
            end
    
            return disks
        end
    
        def _locate_deployement()
            depl = []
            patt = ".*\/datastores\/.*\/#{@id}\/deployment.[0-9]"
            Find.find(ONEROOT) do |path|
                if path =~ /#{patt}/
                    depl.append(path)
                end
            end
            return depl
        end
    
        def state
            @vm.info
            return @info["VM"]["STATE"]
        end

        def status
            @vm.info
            return @vm.status
        end
    
        def _wait_for_it(st)
            cnt = 0
            sleep(1)
            while self.status != st
                cnt += 1
                sleep(1)
                if cnt == 120
                    break
                end
            end
        end
    
        # Aim: Backup a virtual machine
        # Param: none
        # Return: backup status as an int (0 success 1 or more failure)
        def backup()
            bck_root = "#{BACKUP_ROOT}/one"
            dest_dir = ""
            vm_path = ""
    
            @vm.suspend()
            self._wait_for_it("susp")

            # VM is suspended so we can rsync all files from the VM root
            pth = Pathname.new(File.expand_path(self.vm_path))
            rlv = pth.relative_path_from(Pathname.new(ONEROOT)).to_s
            dest_dir="#{bck_root}/#{File.dirname(rlv)}"
            puts(" * #{dest_dir}")
            Rsync.run(self.vm_path, dest_dir, "-aP --delete-after") do |result|
                if result.success?
                    result.changes.each do |change|
                        puts "\t - #{change.filename} (#{change.summary})"
                    end
                else
                    puts result.error
                end
            end

            # Backup symlink targets
            Find.find(@vm_path) do |path|
                if File.symlink?(path)
                    target = File.readlink(path)
                    dest_dir = "#{bck_root}/#{Pathname.new(File.expand_path(target)).relative_path_from(Pathname.new(ONEROOT)).to_s}"
                    puts(" * #{dest_dir}")
                    Rsync.run(target, dest_dir, "-aP --delete-after") do |result|
                        if result.success?
                            result.changes.each do |change|
                                puts "\t - /#{change.filename} (#{change.summary})"
                            end
                        else
                            puts result.error
                        end
                    end
                end
            end
    
            # Backup context disk
            if @context_disk_id
                pth = Pathname.new(File.expand_path(@vm_path))
                rlv = pth.relative_path_from(Pathname.new(ONEROOT))
                imFile = "#{@vm_path}/disk.#{@context_disk_id}"
                dstFile = "#{bck_root}/#{rlv}/disk.#{@context_disk_id}"
                sum = Digest::SHA1.file(imFile)
                if not One::Utils.copyImage(imFile, dstFile, sum, compress=false)
                    return 1
                end
            end
    
            @vm.resume()
            return 0
        end
    
        def show()
            puts "Virtual machine: "
            puts "    ID: #{@id}"
            puts "  NAME: #{@name}"
            puts " STATE: #{@state}"
            puts "LABELS:"
            @labels.each do |lb|
                puts("  - #{lb}")
            end
            puts "  DISK: "
            @disks.each do |ds|
                ds.show()
            end
        end
    end
  end
end
